@@ -1,9 +1,10 @@
 """
 Wrapper around scrypt.
 """
+from json import dumps, loads
 from os import urandom
 from scrypt import hash
-from twisted.internet import reactor
+from twisted.internet import defer, reactor
 from twisted.internet.threads import deferToThreadPool
 from twisted.python import threadpool
 
@@ -11,11 +12,11 @@ from twisted.python import threadpool
 class Wrapper(object):
     urandom = staticmethod(urandom)
 
-    def __init__(self, reactor, threadPool, saltLength, **kwargs):
+    def __init__(self, reactor, threadPool, saltLength, **params):
         self.reactor = reactor
         self.threadPool = threadPool
         self.saltLength = saltLength
-        self.kwargs = kwargs
+        self._params = params
 
 
     def _deferToThread(self, f, *a, **kw):
@@ -33,24 +34,27 @@ class Wrapper(object):
         return deferToThreadPool(self.reactor, self.threadPool, f, *a, **kw)
 
 
-    def _hash(self, password):
-        """Computes the hash of the given password, with the wrapper's
-        parameters.
-
-        """
-        salt = self.urandom(self.saltLength)
-        return self._deferToThread(hash, password, salt, **self.kwargs)
-
-
     def checkPassword(self, stored, provided):
         """Checks that the stored key was computed from the provided password.
 
         Returns a deferred that will fire with ``True``, if the
-        password was correct, or ``False`` otherwise.
+        password was correct, or ``False`` otherwise. The deferred
+        will fail with ValueError if the stored value was not
+        recognized as a txscrypt value.
 
         """
-        d = self._hash(provided)
-        return d.addCallback(stored.decode("base64").__eq__)
+        try:
+            comment, encodedParams, encodedKey, encodedSalt = stored.split("$")
+        except ValueError:
+            return defer.fail(ValueError("Invalid number of fields"))
+
+        if comment != "txscrypt":
+            return defer.fail(ValueError("Missing txscrypt prefix"))
+
+        params = loads(encodedParams)
+        key, salt = [s.decode("base64") for s in [encodedKey, encodedSalt]]
+        d = self._deferToThread(hash, provided, salt, **params)
+        return d.addCallback(key.__eq__)
 
 
     def computeKey(self, password):
@@ -58,14 +62,17 @@ class Wrapper(object):
         function.
 
         """
-        return self._hash(password).addCallback(self._encode)
+        salt = self.urandom(self.saltLength)
+        d = self._deferToThread(hash, password, salt, **self._params)
+        return d.addCallback(self._encode, salt)
 
 
-    def _encode(self, computedKey):
+    def _encode(self, key, salt):
+        """Encodes the computed key, salt and parameters.
+
         """
-        Base64-encodes the key.
-        """
-        return computedKey.encode("base64").strip()
+        key, salt = [s.encode("base64").strip() for s in [key, salt]]
+        return "txscrypt${0}${1}${2}".format(dumps(self._params), key, salt)
 
 
 
